%debug
%error-verbose
%locations
%define api.pure full

%code top
{
# include <stdio.h>
# include <stdlib.h>
}

%code provides
{
  /* Generated after the token definition. */
#define YY_DECL enum yytokentype yylex(YYSTYPE* yylval, YYLTYPE* yylloc)
  YY_DECL;
  extern unsigned nerrs;
  /* Prologue. */
  void yyerror (YYLTYPE* yylloc, const char* msg);
}

%define api.token.prefix {TOK_}
%token PLUS "+" MINUS "-"
       STAR "*" SLASH "/"
       LPAREN "(" RPAREN ")"
       EOL "\n"
%token EOF 0

%union
 {
   int intval;
   float floatval;
 };

%token <intval> INT "number"
%type <intval> exp
%printer { fprintf (stderr, "%d", $$); } <intval>

%left PLUS MINUS
%left STAR SLASH

%%
input:
  %empty
| input line 
;

line:
  exp   "\n" { printf ("%d\n", $1); }
| error "\n"
;

exp:
  exp "+" exp  { $$ = $1 + $3; }
| exp "-" exp  { $$ = $1 - $3; }
| exp "*" exp  { $$ = $1 * $3; }
| exp "/" exp  { if ($3 == 0)
                   {
                     yyerror(&@3, "invalid division");
                     YYERROR;
                   }
                   else
                     $$ = $1 / $3;
                 }
| "number"     { $$ = $1; }
| "(" exp ")"  { $$ = $2; }
| "+" exp      { $$ = $2; }
| "-" exp      { $$ = -$2; }
;
%%
unsigned nerrs;

     /* Epilogue. */
void
yyerror (YYLTYPE* yylloc, const char* msg)
{
  ++nerrs;
  YY_LOCATION_PRINT(stderr, *yylloc);
  fprintf (stderr, ": %s\n", msg);
}

int
main(void)
{
  nerrs = 0;
  yydebug = !!getenv("YYDEBUG");
  yyparse();
  return !!nerrs;
}
